﻿// ********************************************************************************************************
// <copyright file="BOMapWindow.cs" company="TopX Geo-ICT">
//     Copyright (c) 2012 TopX Geo-ICT. All rights reserved.
// </copyright>
// ********************************************************************************************************
// The contents of this file are subject to the Mozilla Public License Version 1.1 (the "License"); 
// you may not use this file except in compliance with the License. You may obtain a copy of the License at 
// http:// www.mozilla.org/MPL/ 
// Software distributed under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF 
// ANY KIND, either express or implied. See the License for the specific language governing rights and 
// limitations under the License. 
// 
// The Initial Developer of this version is Jeen de Vegt.
// 
// Contributor(s): (Open source contributors should list themselves and their modifications here). 
// Change Log: 
// Date           Changed By      Notes
// 29 March 2012  Jeen de Vegt    Initial coding
// ********************************************************************************************************
namespace GridColorAnalyser.BO
{
    using System;
    using System.IO;
    using System.Windows.Forms;
    using MapWindow.Interfaces;
    using MapWinGIS;

    /// <summary>
    ///   Takes care of the communication with the MapWindow-object
    /// </summary>
    public class BOMapWindow
    {
        /// <summary>The MapWindow-object</summary>
        private IMapWin mapWin;

        /// <summary>Flag indicating if the map is being refreshed</summary>
        private bool refreshing = false;

       /// <summary>Is the map refreshing?</summary>
        public bool Refreshing
           {
           get { return this.refreshing; }
           }

        /// <summary>Initializes a new instance of the BOMapWindow class</summary>
        /// <param name = "app">The name of ImapWin.</param>
        public BOMapWindow(IMapWin app)
        {
            this.mapWin = app;
        }

        /// <summary>
        /// Creates a suitable image representation of the specified grid.
        /// </summary>
        private Image GetImageRep (Grid grid, GridColorScheme gcs)
           {
           Utils utils = new Utils ();
           Image image = utils.GridToImage (grid, gcs, null);

           // Save the image to the disk.
           string imgName = Path.ChangeExtension (grid.Filename, ".bmp");
           image.Save (imgName, true, MapWinGIS.ImageType.BITMAP_FILE, null);
           image.Close ();

           // Open from the disk so that the image is not in RAM.
           image.Open (imgName, MapWinGIS.ImageType.BITMAP_FILE, false, null);

           // Update the transparency colors for the image.
           image.TransparencyColor = gcs.NoDataColor;
           image.TransparencyColor2 = gcs.NoDataColor;
           image.UseTransparencyColor = true;
           return image;
           }

        /// <summary>Updates the map with the specified color scheme.</summary>
        public void UpdateMap (BOGridFile bogrid, GridColorScheme gcs)
           {
           try
              {
              Image image = GetImageRep (bogrid.GridFile, gcs);

              // Failed to create the image representation of the grid?
              if (image == null)
                 return;

              string gridFilename = bogrid.GridFile.Filename;
              int group = this.mapWin.View.LegendControl.Layers.GroupOf (this.mapWin.Layers.CurrentLayer);

              // Note that we are updating the layers.
              this.refreshing = true;

              // Replace the existing grid layer with our new one.
              this.mapWin.View.LegendControl.Layers.Remove (this.mapWin.Layers.CurrentLayer);
              int handle = this.mapWin.View.LegendControl.Layers.Add (image, true, group);

              // Update the handle reference on the BO grid.
              bogrid.LayerHandle = handle;

              // Update the properties of the new layer.
              this.mapWin.View.LegendControl.Map.set_GridFileName (handle, gridFilename);
              this.mapWin.View.LegendControl.Map.set_LayerName (handle, bogrid.LayerName);
              this.mapWin.View.LegendControl.Map.SetImageLayerColorScheme (handle, gcs);
              this.mapWin.View.LegendControl.Layers.ItemByHandle(handle).Type = MapWindow.Interfaces.eLayerType.Grid;
              this.mapWin.View.LegendControl.Layers.ItemByHandle(handle).Refresh();
                    
              // Refresh the map.
              this.mapWin.View.LegendControl.Refresh ();
              this.mapWin.View.Redraw ();
              }
           catch (Exception)
              {
              }
           finally
              {
              this.refreshing = false;
              }
           }

        /// <summary>Moves to the selected shape(s)</summary>
        /// <param name = "selectedRows">A collection of th selected rows.</param>
        /// <param name = "shapeFile">The current shapefile.</param>
        public void MoveToSelected(DataGridViewSelectedRowCollection selectedRows, Shapefile shapeFile)
        {
            // TODO: Check if this is the most optimal method. Perhaps the SF objects has some better solutions
            try
            {
                MapWinGIS.Extents currentEx = this.mapWin.View.Extents;

                // Get the current center of the view
                double mpX = (currentEx.xMax - currentEx.xMin) / 2;
                double mpY = (currentEx.yMax - currentEx.yMin) / 2;

                // Determine the center point of the selected shapes
                double centroidX = shapeFile.get_Shape(Convert.ToInt32(selectedRows[0].Cells[0].Value)).Center.x;
                double centroidY = shapeFile.get_Shape(Convert.ToInt32(selectedRows[0].Cells[0].Value)).Center.y;

                // Check if the new centroid is calculated OK
                if (centroidX * centroidY != 0)
                {
                    // Create new extents
                    MapWinGIS.Extents newEx = new MapWinGIS.Extents();

                    // Set new extents
                    newEx.SetBounds((centroidX - mpX), (centroidY - mpY), 0, (centroidX + mpX), (centroidY + mpY), 0);

                    this.mapWin.View.Extents = newEx;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(string.Format("Error in MoveToSelected: {0}", ex.Message));
            }
        }

        /// <summary>Zoom to the selected shape(s)</summary>
        /// <param name = "selectedRows">A collection of the selected rows.</param>
        /// <param name = "shapeFile">The current shapefile.</param>
        public void ZoomToSelected(DataGridViewSelectedRowCollection selectedRows, Shapefile shapeFile)
        {
            try
            {
                double maxX = double.MinValue;
                double maxY = double.MinValue;
                double minX = double.MaxValue;
                double minY = double.MaxValue;

                this.mapWin.Layers[this.mapWin.Layers.CurrentLayer].Visible = true;

                foreach (DataGridViewRow row in selectedRows)
                {
                    MapWinGIS.Extents ext = shapeFile.QuickExtents(Convert.ToInt32(row.Cells[0].Value));

                    maxX = ext.xMax > maxX ? ext.xMax : maxX;
                    maxY = ext.yMax > maxY ? ext.yMax : maxY;
                    minX = ext.xMin < minX ? ext.xMin : minX;
                    minY = ext.yMin < minY ? ext.yMin : minY;
                }

                // Pad extents now
                double dx = this.PadExtentsSelected(maxX, minX);
                maxX = maxX + dx;
                minX = minX - dx;

                double dy = this.PadExtentsSelected(maxY, minY);
                maxY = maxY + dy;
                minY = minY - dy;

                MapWinGIS.Extents exts = new Extents();

                if (this.mapWin.View.SelectedShapes.NumSelected == 1 && this.mapWin.Layers[this.mapWin.Layers.CurrentLayer].LayerType == eLayerType.PointShapefile)
                {
                    MapWinGIS.Shapefile sf = this.mapWin.Layers[this.mapWin.Layers.CurrentLayer] as Shapefile;

                    double xpad = (1 / 100) * (sf.Extents.xMax - sf.Extents.xMin);
                    double ypad = (1 / 100) * (sf.Extents.yMax - sf.Extents.yMin);

                    exts.SetBounds(minX + xpad, minY - ypad, 0, maxX - xpad, maxY + ypad, 0);
                }
                else
                {
                    exts.SetBounds(minX, minY, 0, maxX, maxY, 0);
                }

                this.mapWin.View.Extents = exts;
            }
            catch (Exception ex)
            {
                MessageBox.Show(string.Format("Error in ZoomToSelected: {0}", ex.Message));
            }
        }

        /// <summary>Zoom to the shape dat is currently selected/edited</summary>
        /// <param name = "shapeId">Id of the selected shape.</param>
        /// <param name = "shapeFile">The current shapefile.</param>
        public void ZoomToEdit(int shapeId, Shapefile shapeFile)
        {
            try
            {
                MapWinGIS.Shape shape = shapeFile.get_Shape(shapeId);

                double maxX = shape.Extents.xMax;
                double maxY = shape.Extents.yMax;
                double minX = shape.Extents.xMin;
                double minY = shape.Extents.yMin;

                this.mapWin.Layers[this.mapWin.Layers.CurrentLayer].Visible = true;

                // Pad extents now
                double dx = this.PadExtentsEdit(maxX, minX);
                maxX += dx;
                minX -= dx;

                double dy = this.PadExtentsEdit(maxY, minY);
                maxY += dy;
                minY -= dy;

                MapWinGIS.Extents exts = new MapWinGIS.Extents();
                exts.SetBounds(minX, minY, 0, maxX, maxY, 0);

                this.mapWin.View.Extents = exts;
            }
            catch (Exception ex)
            {
                MessageBox.Show(string.Format("Error: {0}", ex.Message));
            }
        }

        /// <summary>Save shapfileData to a shapefile</summary>
        /// <param name = "shapeFile">The current shapefile.</param>
        public void ExportShapes(Shapefile shapeFile)
        {
            if (this.mapWin.Layers.NumLayers == 0)
            {
                MessageBox.Show("No layer selected.");
                return;
            }

            if (shapeFile.NumSelected == 0)
            {
                MessageBox.Show("No shapes selected.");
                return;
            }

            SaveFileDialog saveFileDialog = new SaveFileDialog
                {
                    Filter = @"Shapefiles (*.shp)|*.shp", 
                    FilterIndex = 2, 
                    RestoreDirectory = true
                };

            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                bool loadShapeFile = MessageBox.Show(@"Do you want to load the new shapefile?", @"TableEditor", MessageBoxButtons.YesNo) == DialogResult.Yes;

                // TODO: Use new export method of SF object:
                MapWinGeoProc.Selection.ExportSelectedMWViewShapes(this.mapWin, saveFileDialog.FileName, loadShapeFile);
            }
        }

        /// <summary>Set Exent for selected</summary>
        /// <param name = "max">Max value.</param>
        /// <param name = "min">Min value.</param>
        /// <returns>The result extent</returns>
        private double PadExtentsSelected(double max, double min)
        {
            double dx = max - min;

            dx /= 8;

            return dx == 0 ? 1 : dx;
        }

        /// <summary>Set Exent for edit</summary>
        /// <param name = "max">Max value.</param>
        /// <param name = "min">Min value.</param>
        /// <returns>The result extent</returns>
        private double PadExtentsEdit(double max, double min)
        {
            double dx = max - min;

            dx *= this.mapWin.View.ExtentPad;

            return dx == 0 ? 1 : dx;
        }
    }
}
