﻿// ********************************************************************************************************
// <copyright file="Main.cs" company="TopX Geo-ICT">
//     Copyright (c) 2012 TopX Geo-ICT. All rights reserved.
// </copyright>
// ********************************************************************************************************
// The contents of this file are subject to the Mozilla Public License Version 1.1 (the "License"); 
// you may not use this file except in compliance with the License. You may obtain a copy of the License at 
// http:// www.mozilla.org/MPL/ 
// Software distributed under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF 
// ANY KIND, either express or implied. See the License for the specificlanguage governing rights and 
// limitations under the License. 
// 
// The Initial Developer of this version is Maurits van Ravesteijn.
// 
// Contributor(s): (Open source contributors should list themselves and their modifications here). 
// Change Log: 
// Date           Changed By              Notes
//    Aug 2012    Maurits van Ravesteijn  GridColorAnalyser
// March 2013     Brad Hester             Fixed some issues as asked by Maurits
// April 1 2013   Paul Meems              Made it ready for the build scripts
// ********************************************************************************************************
namespace GridColorAnalyser
{
  using System;
  using System.Collections.Generic;
  using System.Windows.Forms;
  using GridColorAnalyser.BO;
  using GridColorAnalyser.Forms;
  using MapWindow.Interfaces;
  using MapWinGIS;

  /// <summary>
  ///   This is the start of the plug-in. In here all methods of the IPlugin interface can be implemented.
  /// </summary>
  public class Main : PluginBase
  {
    /// <summary>Change this to your own address</summary>
    private const string ReportEmailaddress = "mhraaf@gmail.com";

    /// <summary>The stack with the toolbuttons</summary>
    private readonly Stack<string> addedToolbuttons = new Stack<string>();

    /// <summary>The name of your button. This need to be a constant</summary>
    private const string MyButton = "Grid Color Analyser";

    /// <summary>The name of the toolbar</summary>
    private string toolbarName;

    /// <summary>Reference to the frmTableEditor</summary>
    private FormGraph graphColorAnalyser;

    /// <summary>The grid we're working on</summary>
    private Grid grid;

    /// <summary>This event is called when a plugin is loaded or turned on in the MapWindow.</summary>
    public override void Initialize()
    {
      try
      {
        // Create toolbar:
        this.toolbarName = this.Author;
        this.CreateToolbar();
      }
      catch (Exception ex)
      {
        this.App.ShowErrorDialog(ex, ReportEmailaddress);
      }
    }

    /// <summary>This method is called when a plugin is unloaded. The plugin should remove all toolbars, buttons and menus that it added.</summary>
    public override void Terminate()
    {
      // Remove toolbar:
      if (this.addedToolbuttons == null)
      {
        return;
      }

      // Unloads the buttons in reverse order
      while (this.addedToolbuttons.Count > 0)
      {
        this.App.Toolbar.RemoveButton(this.addedToolbuttons.Pop());
      }

      // If all buttons are removed, remove toolbar as well:
      if (this.App.Toolbar.NumToolbarButtons(this.toolbarName) == 0)
      {
        this.App.Toolbar.RemoveToolbar(this.toolbarName);
      }
    }

    /// <summary>Occurs when a user clicks on a toolbar button or menu item.</summary>
    /// <param name = "itemName">The name of the item clicked on.</param>
    /// <param name = "handled">Reference parameter.  Setting Handled to true prevents other plugins from receiving this event.</param>
    public override void ItemClicked(string itemName, ref bool handled)
    {
      try
      {
        switch (itemName)
        {
          case MyButton:

            if (this.App.Layers.NumLayers == 0)
            {
              MessageBox.Show("No grid layers loaded. Do that first.");
              return;
            }

            if (this.graphColorAnalyser != null &&
               !this.graphColorAnalyser.IsDisposed)
            {
              this.graphColorAnalyser.BringToFront();
              return;
            }

            BOGridFile gr = this.CreateBOGridFile();

            if (gr == null)
            {
              MessageBox.Show("Selected layer is not a grid.");
              return;
            }

            this.graphColorAnalyser = new FormGraph(gr, new BOMapWindow(this.App));
            this.graphColorAnalyser.Show();
            handled = true;
            break;
        }
      }
      catch (Exception ex)
      {
        this.App.ShowErrorDialog(ex, ReportEmailaddress);
      }
    }

    /// <summary>New layer selected.</summary>
    /// <param name = "handle">The handle of the newly selected layer.</param>
    public override void LayerSelected(int handle)
    {
      if (this.graphColorAnalyser != null)
      {
        this.graphColorAnalyser.RespondToLayerSelected(handle);
      }

      base.LayerSelected(handle);
    }

    /// <summary>
    /// Layer removed.
    /// </summary>
    /// <param name="handle">The handle of the removed layer.</param>
    public override void LayerRemoved(int handle)
    {
      if (this.graphColorAnalyser != null)
      {
        this.graphColorAnalyser.RespondToLayerRemoved(handle);
      }

      base.LayerRemoved(handle);
    }

    /// <summary>
    /// Layers have been cleared.
    /// </summary>
    public override void LayersCleared()
    {
      if (this.graphColorAnalyser != null)
      {
        this.graphColorAnalyser.RespondToLayerSelected(-1);
      }

      base.LayersCleared();
    }

    /// <summary>Create the toolbar</summary>
    private void CreateToolbar()
    {
      // Add toolbar with buttons:
      var t = this.App.Toolbar;
      t.AddToolbar(this.toolbarName);

      // First button:
      var b = t.AddButton(MyButton, this.toolbarName, string.Empty, string.Empty);
      b.BeginsGroup = true;
      b.Tooltip = "Grid Color Analyser";
      b.Category = this.toolbarName;
      b.Text = "Grid Color\nAnalyser";

      b.Picture = Properties.Resources.GCA.ToBitmap();

      // Save to remove it easily on terminate:
      this.addedToolbuttons.Push(MyButton);
    }

    /// <summary>Creates the BOShapeFile.</summary>
    /// <returns>The BOShapeFile</returns>
    private BOGridFile CreateBOGridFile()
    {
      BOGridFile boGridFile = null;
      try
      {
        var current = this.App.Layers[App.Layers.CurrentLayer];
        this.grid = current.GetGridObject;

        if (this.grid != null)
        {
          boGridFile = new BOGridFile
              {
                GridFile = this.grid,
                LayerName = current.Name,
                LayerHandle = App.Layers.CurrentLayer
              };
        }
      }
      catch (Exception e)
      {
        MessageBox.Show(e.Message);
      }

      return boGridFile;
    }
  }
}
