﻿// ********************************************************************************************************
// <copyright file="Utils.cs" company="TopX Geo-ICT">
//     Copyright (c) 2011-2013 TopX Geo-ICT. All rights reserved.
// </copyright>
// ********************************************************************************************************
// The contents of this file are subject to the Mozilla Public License Version 1.1 (the "License"); 
// you may not use this file except in compliance with the License. You may obtain a copy of the License at 
// http:// www.mozilla.org/MPL/ 
// Software distributed under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF 
// ANY KIND, either express or implied. See the License for the specific language governing rights and 
// limitations under the License. 
// 
// The Initial Developer of this version is Paul Meems.
// 
// Contributor(s): (Open source contributors should list themselves and their modifications here). 
// Change Log: 
// Date           Changed By      Notes
// 24 Nov 2011    Paul Meems      Inital coding
// 09 Sept 2013   Paul Meems      Converted to a template plug-in for general use
// ********************************************************************************************************
namespace PluginTemplate.Classes
{
  using System;
  using System.Linq;
  using System.Windows.Forms;

  /// <summary>Class with all kinds of usefull methods</summary>
  public static class Utils
  {
    /// <summary>
    /// Searches for a form that matches the type and returns the first 
    /// occurence with the application's open forms 
    /// </summary>
    /// <typeparam name="T">The type of the form to look for</typeparam>
    /// <param name="parentHandle">Used to not let the forms fall behind MapWindow </param>
    /// <param name="loadIfClosed">Should the form be loaded if it doesn't exists</param>
    /// <returns>Returns the form when found, returns the new instance specified by newinstance otherwise</returns>
    public static T GetActivatedWindow<T>(IntPtr parentHandle, bool loadIfClosed) where T : Form, new()
    {
      var f = Application.OpenForms.OfType<T>().FirstOrDefault();

      // loadIfClosed is false when called from the Terminate method:
      if (loadIfClosed)
      {
        if (f == null)
        {
          f = new T();
          f.Location = GetFormLocation(parentHandle, f.Width, f.Height);
          f.Show(Control.FromHandle(parentHandle));
        }
        else
        {
          f.Visible = true;
          f.Activate();
        }
      }

      return f;
    }

    /// <summary>
    ///   Fills the combobox with all visible layers in the map using LINQ
    /// </summary>
    /// <param name = "frm">The form to work with</param>
    public static void FillVisibleLayers(Forms.Layers frm)
    {
      // C#3.0: LINQ to Entities: Language-Integrated Query:            
      var visibleLayers = from l in Helper.MapWin.Layers
                          where l.Visible
                          orderby l.GlobalPosition descending
                          select l;

      frm.cboLayers.DisplayMember = "Name";
      frm.cboLayers.ValueMember = "Handle";
      frm.cboLayers.DataSource = visibleLayers.ToArray();
      frm.cboLayers.SelectedValue = Helper.MapWin.Layers.CurrentLayer;
    }

    /// <summary>Calculates the location of the child form, based on the center of the parent</summary>
    /// <param name="parentHandle">The parent handle</param>
    /// <param name="childWidth">The child width</param>
    /// <param name="childHeight">The child height</param>
    /// <returns>The location</returns>
    private static System.Drawing.Point GetFormLocation(IntPtr parentHandle, int childWidth, int childHeight)
    {
      var parentForm = Control.FromHandle(parentHandle);
      var centerParent = new System.Drawing.Point(
          parentForm.Location.X + (parentForm.Width / 2), parentForm.Location.Y + (parentForm.Height / 2));
      return new System.Drawing.Point(centerParent.X - (childWidth / 2), centerParent.Y - (childHeight / 2));
    }
  }
}
